diff --git a/src/backend/catalog/dependency.c b/src/backend/catalog/dependency.c
index 90b1cd8..08a3ea6 100644
--- a/src/backend/catalog/dependency.c
+++ b/src/backend/catalog/dependency.c
@@ -50,6 +50,7 @@
 #include "catalog/pg_transform.h"
 #include "catalog/pg_trigger.h"
 #include "catalog/pg_ts_config.h"
+#include "catalog/pg_ts_config_map.h"
 #include "catalog/pg_ts_dict.h"
 #include "catalog/pg_ts_parser.h"
 #include "catalog/pg_ts_template.h"
@@ -149,6 +150,7 @@ static const Oid object_classes[] = {
 	TSDictionaryRelationId,		/* OCLASS_TSDICT */
 	TSTemplateRelationId,		/* OCLASS_TSTEMPLATE */
 	TSConfigRelationId,			/* OCLASS_TSCONFIG */
+	TSConfigMapRelationId,		/* OCLASS_TSCONFIG_MAP */
 	AuthIdRelationId,			/* OCLASS_ROLE */
 	DatabaseRelationId,			/* OCLASS_DATABASE */
 	TableSpaceRelationId,		/* OCLASS_TBLSPACE */
@@ -2382,6 +2384,9 @@ getObjectClass(const ObjectAddress *object)
 		case TSConfigRelationId:
 			return OCLASS_TSCONFIG;
 
+		case TSConfigMapRelationId:
+			return OCLASS_TSCONFIG_MAP;
+
 		case AuthIdRelationId:
 			return OCLASS_ROLE;
 
diff --git a/src/backend/catalog/objectaddress.c b/src/backend/catalog/objectaddress.c
index 052aab1..e01caa1 100644
--- a/src/backend/catalog/objectaddress.c
+++ b/src/backend/catalog/objectaddress.c
@@ -48,6 +48,7 @@
 #include "catalog/pg_transform.h"
 #include "catalog/pg_trigger.h"
 #include "catalog/pg_ts_config.h"
+#include "catalog/pg_ts_config_map.h"
 #include "catalog/pg_ts_dict.h"
 #include "catalog/pg_ts_parser.h"
 #include "catalog/pg_ts_template.h"
@@ -390,6 +391,18 @@ static const ObjectPropertyType ObjectProperty[] =
 		true
 	},
 	{
+		TSConfigMapRelationId,
+		TSConfigMapIndexId,
+		TSCONFIGMAP,
+		TSCONFIGNAMENSP,
+		InvalidAttrNumber,
+		InvalidAttrNumber,
+		InvalidAttrNumber,
+		InvalidAttrNumber,
+		ACL_KIND_TSCONFIGURATION,
+		true
+	},
+	{
 		TSDictionaryRelationId,
 		TSDictionaryOidIndexId,
 		TSDICTOID,
@@ -595,6 +608,10 @@ static const struct object_type_map
 	{
 		"text search configuration", OBJECT_TSCONFIGURATION
 	},
+	/* OCLASS_TSCONFIG_MAP */
+	{
+		"text search configuration mapping", OBJECT_TSCONFIG_MAPPING
+	},
 	/* OCLASS_ROLE */
 	{
 		"role", OBJECT_ROLE
@@ -903,6 +920,11 @@ get_object_address(ObjectType objtype, List *objname, List *objargs,
 				address.objectId = get_ts_config_oid(objname, missing_ok);
 				address.objectSubId = 0;
 				break;
+			case OBJECT_TSCONFIG_MAPPING:
+				address.classId = TSConfigMapRelationId;
+				address.objectId = get_ts_config_oid(objname, missing_ok);
+				address.objectSubId = 0;
+				break;
 			case OBJECT_USER_MAPPING:
 				address = get_object_address_usermapping(objname, objargs,
 														 missing_ok);
@@ -2150,6 +2172,7 @@ check_object_ownership(Oid roleid, ObjectType objtype, ObjectAddress address,
 							   NameListToString(objname));
 			break;
 		case OBJECT_TSCONFIGURATION:
+		case OBJECT_TSCONFIG_MAPPING:
 			if (!pg_ts_config_ownercheck(address.objectId, roleid))
 				aclcheck_error(ACLCHECK_NOT_OWNER, ACL_KIND_TSCONFIGURATION,
 							   NameListToString(objname));
@@ -2237,7 +2260,7 @@ get_object_namespace(const ObjectAddress *address)
 int
 read_objtype_from_string(const char *objtype)
 {
-	ObjectType	type;
+	ObjectType	type = -1;
 	int			i;
 
 	for (i = 0; i < lengthof(ObjectTypeMap); i++)
@@ -2905,6 +2928,7 @@ getObjectDescription(const ObjectAddress *object)
 			}
 
 		case OCLASS_TSCONFIG:
+		case OCLASS_TSCONFIG_MAP:
 			{
 				HeapTuple	tup;
 
@@ -3564,6 +3588,10 @@ getObjectTypeDescription(const ObjectAddress *object)
 			appendStringInfoString(&buffer, "text search configuration");
 			break;
 
+		case OCLASS_TSCONFIG_MAP:
+			appendStringInfoString(&buffer, "text search configuration mapping");
+			break;
+
 		case OCLASS_ROLE:
 			appendStringInfoString(&buffer, "role");
 			break;
@@ -4296,6 +4324,7 @@ getObjectIdentityParts(const ObjectAddress *object,
 			}
 
 		case OCLASS_TSCONFIG:
+		case OCLASS_TSCONFIG_MAP:
 			{
 				HeapTuple	tup;
 				Form_pg_ts_config formCfg;
diff --git a/src/backend/commands/seclabel.c b/src/backend/commands/seclabel.c
index 1ef98ce..aa4de97 100644
--- a/src/backend/commands/seclabel.c
+++ b/src/backend/commands/seclabel.c
@@ -63,6 +63,11 @@ ExecSecLabelStmt(SecLabelStmt *stmt)
 					(errcode(ERRCODE_INVALID_PARAMETER_VALUE),
 					 errmsg("must specify provider when multiple security label providers have been loaded")));
 		provider = (LabelProvider *) linitial(label_provider_list);
+		/*
+		 * Set the provider in the statement so that DDL deparse can use
+		 * provider explicitly in generated statement.
+		 */
+		stmt->provider = (char *) provider->provider_name;
 	}
 	else
 	{
diff --git a/src/backend/commands/sequence.c b/src/backend/commands/sequence.c
index 9c1037f..b0f8003 100644
--- a/src/backend/commands/sequence.c
+++ b/src/backend/commands/sequence.c
@@ -1517,6 +1517,40 @@ process_owned_by(Relation seqrel, List *owned_by)
 		relation_close(tablerel, NoLock);
 }
 
+/*
+ * Return sequence parameters, detailed
+ */
+Form_pg_sequence
+get_sequence_values(Oid sequenceId)
+{
+	Buffer		buf;
+	SeqTable	elm;
+	Relation	seqrel;
+	HeapTupleData seqtuple;
+	Form_pg_sequence seq;
+	Form_pg_sequence retSeq;
+
+	retSeq = palloc(sizeof(FormData_pg_sequence));
+
+	/* open and AccessShareLock sequence */
+	init_sequence(sequenceId, &elm, &seqrel);
+
+	if (pg_class_aclcheck(sequenceId, GetUserId(),
+						  ACL_SELECT | ACL_UPDATE | ACL_USAGE) != ACLCHECK_OK)
+		ereport(ERROR,
+				(errcode(ERRCODE_INSUFFICIENT_PRIVILEGE),
+				 errmsg("permission denied for sequence %s",
+						RelationGetRelationName(seqrel))));
+
+	seq = read_seq_tuple(elm, seqrel, &buf, &seqtuple);
+
+	memcpy(retSeq, seq, sizeof(FormData_pg_sequence));
+
+	UnlockReleaseBuffer(buf);
+	relation_close(seqrel, NoLock);
+
+	return retSeq;
+}
 
 /*
  * Return sequence parameters, for use by information schema
diff --git a/src/backend/commands/tablecmds.c b/src/backend/commands/tablecmds.c
index 126b119..54a5ebf 100644
--- a/src/backend/commands/tablecmds.c
+++ b/src/backend/commands/tablecmds.c
@@ -3517,8 +3517,10 @@ ATExecCmd(List **wqueue, AlteredTableInfo *tab, Relation rel,
 			address = CommentObject((CommentStmt *) cmd->def);
 			break;
 		case AT_AddIndexConstraint:		/* ADD CONSTRAINT USING INDEX */
+			EventTriggerInhibitCommandCollection();
 			address = ATExecAddIndexConstraint(tab, rel, (IndexStmt *) cmd->def,
 											   lockmode);
+			EventTriggerUndoInhibitCommandCollection();
 			break;
 		case AT_AlterConstraint:		/* ALTER CONSTRAINT */
 			address = ATExecAlterConstraint(rel, cmd, false, false, lockmode);
@@ -8192,7 +8194,8 @@ ATExecAlterColumnType(AlteredTableInfo *tab, Relation rel,
 				if (!list_member_oid(tab->changedConstraintOids,
 									 foundObject.objectId))
 				{
-					char	   *defstring = pg_get_constraintdef_string(foundObject.objectId);
+					char	   *defstring = pg_get_constraintdef_string(foundObject.objectId,
+																		true);
 
 					/*
 					 * Put NORMAL dependencies at the front of the list and
diff --git a/src/backend/tcop/utility.c b/src/backend/tcop/utility.c
index e81bbc6..b848ec6 100644
--- a/src/backend/tcop/utility.c
+++ b/src/backend/tcop/utility.c
@@ -1454,6 +1454,8 @@ ProcessUtilitySlow(Node *parsetree,
 
 			case T_AlterTSConfigurationStmt:
 				address = AlterTSConfiguration((AlterTSConfigurationStmt *) parsetree);
+				/* commands are stashed in Make/DropConfigurationMapping */
+				commandCollected = true;
 				break;
 
 			case T_AlterTableMoveAllStmt:
diff --git a/src/backend/utils/adt/regproc.c b/src/backend/utils/adt/regproc.c
index 0bfeb5e..5efa5ea 100644
--- a/src/backend/utils/adt/regproc.c
+++ b/src/backend/utils/adt/regproc.c
@@ -43,7 +43,11 @@
 #include "utils/acl.h"
 
 static char *format_operator_internal(Oid operator_oid, bool force_qualify);
-static char *format_procedure_internal(Oid procedure_oid, bool force_qualify);
+static char *format_procedure_internal(Oid procedure_oid, bool force_qualify,
+						  bool args_only);
+static void format_procedure_args_internal(Form_pg_proc procform,
+							   StringInfo buf, bool force_qualify);
+
 static void parseNameAndArgTypes(const char *string, bool allowNone,
 					 List **names, int *nargs, Oid *argtypes);
 
@@ -364,13 +368,36 @@ to_regprocedure(PG_FUNCTION_ARGS)
 char *
 format_procedure(Oid procedure_oid)
 {
-	return format_procedure_internal(procedure_oid, false);
+	return format_procedure_internal(procedure_oid, false, false);
 }
 
 char *
 format_procedure_qualified(Oid procedure_oid)
 {
-	return format_procedure_internal(procedure_oid, true);
+	return format_procedure_internal(procedure_oid, true, false);
+}
+
+/*
+ * format_procedure_args	- converts proc OID to "(args)"
+ */
+char *
+format_procedure_args(Oid procedure_oid, bool force_qualify)
+{
+	StringInfoData buf;
+	HeapTuple	proctup;
+	Form_pg_proc procform;
+
+	proctup = SearchSysCache1(PROCOID, ObjectIdGetDatum(procedure_oid));
+	if (!HeapTupleIsValid(proctup))
+		elog(ERROR, "cache lookup failed for procedure %u", procedure_oid);
+	procform = (Form_pg_proc) GETSTRUCT(proctup);
+
+	initStringInfo(&buf);
+	format_procedure_args_internal(procform, &buf, force_qualify);
+
+	ReleaseSysCache(proctup);
+
+	return buf.data;
 }
 
 /*
@@ -381,7 +408,7 @@ format_procedure_qualified(Oid procedure_oid)
  * qualified if the function is not in path.
  */
 static char *
-format_procedure_internal(Oid procedure_oid, bool force_qualify)
+format_procedure_internal(Oid procedure_oid, bool force_qualify, bool args_only)
 {
 	char	   *result;
 	HeapTuple	proctup;
@@ -392,8 +419,6 @@ format_procedure_internal(Oid procedure_oid, bool force_qualify)
 	{
 		Form_pg_proc procform = (Form_pg_proc) GETSTRUCT(proctup);
 		char	   *proname = NameStr(procform->proname);
-		int			nargs = procform->pronargs;
-		int			i;
 		char	   *nspname;
 		StringInfoData buf;
 
@@ -401,29 +426,24 @@ format_procedure_internal(Oid procedure_oid, bool force_qualify)
 
 		initStringInfo(&buf);
 
-		/*
-		 * Would this proc be found (given the right args) by regprocedurein?
-		 * If not, or if caller requests it, we need to qualify it.
-		 */
-		if (!force_qualify && FunctionIsVisible(procedure_oid))
-			nspname = NULL;
-		else
-			nspname = get_namespace_name(procform->pronamespace);
-
-		appendStringInfo(&buf, "%s(",
-						 quote_qualified_identifier(nspname, proname));
-		for (i = 0; i < nargs; i++)
+		if (!args_only)
 		{
-			Oid			thisargtype = procform->proargtypes.values[i];
-
-			if (i > 0)
-				appendStringInfoChar(&buf, ',');
-			appendStringInfoString(&buf,
-								   force_qualify ?
-								   format_type_be_qualified(thisargtype) :
-								   format_type_be(thisargtype));
+			/*
+			 * Would this proc be found (given the right args) by
+			 * regprocedurein?  If not, or if caller requests it, we need to
+			 * qualify it.
+			 */
+			if (!force_qualify && FunctionIsVisible(procedure_oid))
+				nspname = NULL;
+			else
+				nspname = get_namespace_name(procform->pronamespace);
+
+			appendStringInfo(&buf, "%s",
+							 quote_qualified_identifier(nspname, proname));
 		}
-		appendStringInfoChar(&buf, ')');
+
+		/* add the attributes */
+		format_procedure_args_internal(procform, &buf, force_qualify);
 
 		result = buf.data;
 
@@ -440,6 +460,33 @@ format_procedure_internal(Oid procedure_oid, bool force_qualify)
 }
 
 /*
+ * Append the parenthised arguments of the given pg_proc row into the output
+ * buffer.  force_qualify indicates whether to schema-qualify type names
+ * regardless of visibility.
+ */
+static void
+format_procedure_args_internal(Form_pg_proc procform, StringInfo buf,
+							   bool force_qualify)
+{
+	int			i;
+	int			nargs = procform->pronargs;
+
+	appendStringInfoChar(buf, '(');
+	for (i = 0; i < nargs; i++)
+	{
+		Oid			thisargtype = procform->proargtypes.values[i];
+
+		if (i > 0)
+			appendStringInfoChar(buf, ',');
+		appendStringInfoString(buf,
+							   force_qualify ?
+							   format_type_be_qualified(thisargtype) :
+							   format_type_be(thisargtype));
+	}
+	appendStringInfoChar(buf, ')');
+}
+
+/*
  * Output an objname/objargs representation for the procedure with the
  * given OID.  If it doesn't exist, an error is thrown.
  *
diff --git a/src/backend/utils/adt/ruleutils.c b/src/backend/utils/adt/ruleutils.c
index 51391f6..19ce290 100644
--- a/src/backend/utils/adt/ruleutils.c
+++ b/src/backend/utils/adt/ruleutils.c
@@ -289,9 +289,6 @@ bool		quote_all_identifiers = false;
  * as a parameter, and append their text output to its contents.
  * ----------
  */
-static char *deparse_expression_pretty(Node *expr, List *dpcontext,
-						  bool forceprefix, bool showimplicit,
-						  int prettyFlags, int startIndent);
 static char *pg_get_viewdef_worker(Oid viewoid,
 					  int prettyFlags, int wrapColumn);
 static char *pg_get_triggerdef_worker(Oid trigid, bool pretty);
@@ -415,8 +412,6 @@ static void get_from_clause_coldeflist(RangeTblFunction *rtfunc,
 						   deparse_context *context);
 static void get_tablesample_def(TableSampleClause *tablesample,
 					deparse_context *context);
-static void get_opclass_name(Oid opclass, Oid actual_datatype,
-				 StringInfo buf);
 static Node *processIndirection(Node *node, deparse_context *context,
 				   bool printit);
 static void printSubscripts(ArrayRef *aref, deparse_context *context);
@@ -428,7 +423,6 @@ static char *generate_function_name(Oid funcid, int nargs,
 					   ParseExprKind special_exprkind);
 static char *generate_operator_name(Oid operid, Oid arg1, Oid arg2);
 static text *string_to_text(char *str);
-static char *flatten_reloptions(Oid relid);
 
 #define only_marker(rte)  ((rte)->inh ? "" : "ONLY ")
 
@@ -461,6 +455,77 @@ pg_get_ruledef_ext(PG_FUNCTION_ARGS)
 	PG_RETURN_TEXT_P(string_to_text(pg_get_ruledef_worker(ruleoid, prettyFlags)));
 }
 
+/*
+ * Given a pair of Datum corresponding to a rule's pg_rewrite.ev_qual and
+ * ev_action columns, return their text representation; ev_qual as a single
+ * string in whereClause and ev_action as a List of strings (which might be
+ * NIL, signalling NOTHING) in actions.
+ */
+void
+pg_get_ruledef_detailed(Datum ev_qual, Datum ev_action,
+						char **whereClause, List **actions)
+{
+	int prettyFlags = 0;
+	char *qualstr = TextDatumGetCString(ev_qual);
+	char *actionstr = TextDatumGetCString(ev_action);
+	List *actionNodeList = (List *) stringToNode(actionstr);
+	StringInfoData buf;
+
+	initStringInfo(&buf);
+	if (strlen(qualstr) > 0 && strcmp(qualstr, "<>") != 0)
+	{
+		Node	   *qual;
+		Query	   *query;
+		deparse_context context;
+		deparse_namespace dpns;
+
+		qual = stringToNode(qualstr);
+
+		query = (Query *) linitial(actionNodeList);
+		query = getInsertSelectQuery(query, NULL);
+
+		AcquireRewriteLocks(query, false, false);
+
+		context.buf = &buf;
+		context.namespaces = list_make1(&dpns);
+		context.windowClause = NIL;
+		context.windowTList = NIL;
+		context.varprefix = (list_length(query->rtable) != 1);
+		context.prettyFlags = prettyFlags;
+		context.wrapColumn = WRAP_COLUMN_DEFAULT;
+		context.indentLevel = PRETTYINDENT_STD;
+
+		set_deparse_for_query(&dpns, query, NIL);
+
+		get_rule_expr(qual, &context, false);
+
+		*whereClause = pstrdup(buf.data);
+	}
+	else
+		*whereClause = NULL;
+
+	if (list_length(actionNodeList) == 0)
+		*actions = NIL;
+	else
+	{
+		ListCell *cell;
+		List	*output = NIL;
+
+		foreach(cell, actionNodeList)
+		{
+			Query	*query = (Query *) lfirst(cell);
+
+			if (query->commandType == CMD_NOTHING)
+				continue;
+
+			resetStringInfo(&buf);
+			get_query_def(query, &buf, NIL, NULL,
+						  prettyFlags, WRAP_COLUMN_DEFAULT, 0);
+			output = lappend(output, pstrdup(buf.data));
+		}
+		*actions = output;
+	}
+}
 
 static char *
 pg_get_ruledef_worker(Oid ruleoid, int prettyFlags)
@@ -611,6 +676,13 @@ pg_get_viewdef_name_ext(PG_FUNCTION_ARGS)
 	PG_RETURN_TEXT_P(string_to_text(pg_get_viewdef_worker(viewoid, prettyFlags, WRAP_COLUMN_DEFAULT)));
 }
 
+char *
+pg_get_viewdef_internal(Oid viewoid)
+{
+	return pg_get_viewdef_worker(viewoid, 0, WRAP_COLUMN_DEFAULT);
+}
+
+
 /*
  * Common code for by-OID and by-name variants of pg_get_viewdef
  */
@@ -685,6 +757,21 @@ pg_get_viewdef_worker(Oid viewoid, int prettyFlags, int wrapColumn)
 	return buf.data;
 }
 
+/*
+ * get_createtableas_def	- Get the accompanying query for a CREATE TABLE AS
+ */
+char *
+pg_get_createtableas_def(Query *query)
+{
+	StringInfoData buf;
+
+	initStringInfo(&buf);
+
+	get_query_def(query, &buf, NIL, NULL, 0, 0, 0);
+
+	return buf.data;
+}
+
 /* ----------
  * get_triggerdef			- Get the definition of a trigger
  * ----------
@@ -835,60 +922,12 @@ pg_get_triggerdef_worker(Oid trigid, bool pretty)
 	if (!isnull)
 	{
 		Node	   *qual;
-		char		relkind;
-		deparse_context context;
-		deparse_namespace dpns;
-		RangeTblEntry *oldrte;
-		RangeTblEntry *newrte;
-
-		appendStringInfoString(&buf, "WHEN (");
+		char	   *qualstr;
 
 		qual = stringToNode(TextDatumGetCString(value));
+		qualstr = pg_get_trigger_whenclause(trigrec, qual, pretty);
 
-		relkind = get_rel_relkind(trigrec->tgrelid);
-
-		/* Build minimal OLD and NEW RTEs for the rel */
-		oldrte = makeNode(RangeTblEntry);
-		oldrte->rtekind = RTE_RELATION;
-		oldrte->relid = trigrec->tgrelid;
-		oldrte->relkind = relkind;
-		oldrte->alias = makeAlias("old", NIL);
-		oldrte->eref = oldrte->alias;
-		oldrte->lateral = false;
-		oldrte->inh = false;
-		oldrte->inFromCl = true;
-
-		newrte = makeNode(RangeTblEntry);
-		newrte->rtekind = RTE_RELATION;
-		newrte->relid = trigrec->tgrelid;
-		newrte->relkind = relkind;
-		newrte->alias = makeAlias("new", NIL);
-		newrte->eref = newrte->alias;
-		newrte->lateral = false;
-		newrte->inh = false;
-		newrte->inFromCl = true;
-
-		/* Build two-element rtable */
-		memset(&dpns, 0, sizeof(dpns));
-		dpns.rtable = list_make2(oldrte, newrte);
-		dpns.ctes = NIL;
-		set_rtable_names(&dpns, NIL, NULL);
-		set_simple_column_names(&dpns);
-
-		/* Set up context with one-deep namespace stack */
-		context.buf = &buf;
-		context.namespaces = list_make1(&dpns);
-		context.windowClause = NIL;
-		context.windowTList = NIL;
-		context.varprefix = true;
-		context.prettyFlags = pretty ? PRETTYFLAG_PAREN | PRETTYFLAG_INDENT : PRETTYFLAG_INDENT;
-		context.wrapColumn = WRAP_COLUMN_DEFAULT;
-		context.indentLevel = PRETTYINDENT_STD;
-		context.special_exprkind = EXPR_KIND_NONE;
-
-		get_rule_expr(qual, &context, false);
-
-		appendStringInfoString(&buf, ") ");
+		appendStringInfo(&buf, "WHEN (%s) ", qualstr);
 	}
 
 	appendStringInfo(&buf, "EXECUTE PROCEDURE %s(",
@@ -929,6 +968,64 @@ pg_get_triggerdef_worker(Oid trigid, bool pretty)
 	return buf.data;
 }
 
+char *
+pg_get_trigger_whenclause(Form_pg_trigger trigrec, Node *whenClause, bool pretty)
+{
+	StringInfoData buf;
+	char		relkind;
+	deparse_context context;
+	deparse_namespace dpns;
+	RangeTblEntry *oldrte;
+	RangeTblEntry *newrte;
+
+	initStringInfo(&buf);
+
+	relkind = get_rel_relkind(trigrec->tgrelid);
+
+	/* Build minimal OLD and NEW RTEs for the rel */
+	oldrte = makeNode(RangeTblEntry);
+	oldrte->rtekind = RTE_RELATION;
+	oldrte->relid = trigrec->tgrelid;
+	oldrte->relkind = relkind;
+	oldrte->alias = makeAlias("old", NIL);
+	oldrte->eref = oldrte->alias;
+	oldrte->lateral = false;
+	oldrte->inh = false;
+	oldrte->inFromCl = true;
+
+	newrte = makeNode(RangeTblEntry);
+	newrte->rtekind = RTE_RELATION;
+	newrte->relid = trigrec->tgrelid;
+	newrte->relkind = relkind;
+	newrte->alias = makeAlias("new", NIL);
+	newrte->eref = newrte->alias;
+	newrte->lateral = false;
+	newrte->inh = false;
+	newrte->inFromCl = true;
+
+	/* Build two-element rtable */
+	memset(&dpns, 0, sizeof(dpns));
+	dpns.rtable = list_make2(oldrte, newrte);
+	dpns.ctes = NIL;
+	set_rtable_names(&dpns, NIL, NULL);
+	set_simple_column_names(&dpns);
+
+	/* Set up context with one-deep namespace stack */
+	context.buf = &buf;
+	context.namespaces = list_make1(&dpns);
+	context.windowClause = NIL;
+	context.windowTList = NIL;
+	context.varprefix = true;
+	context.prettyFlags = pretty ? PRETTYFLAG_PAREN | PRETTYFLAG_INDENT : PRETTYFLAG_INDENT;
+	context.wrapColumn = WRAP_COLUMN_DEFAULT;
+	context.indentLevel = PRETTYINDENT_STD;
+	context.special_exprkind = EXPR_KIND_NONE;
+
+	get_rule_expr(whenClause, &context, false);
+
+	return buf.data;
+}
+
 /* ----------
  * get_indexdef			- Get the definition of an index
  *
@@ -992,6 +1089,8 @@ pg_get_indexdef_columns(Oid indexrelid, bool pretty)
  *
  * This is now used for exclusion constraints as well: if excludeOps is not
  * NULL then it points to an array of exclusion operator OIDs.
+ *
+ * XXX if you change this function, see pg_get_indexdef_detailed too.
  */
 static char *
 pg_get_indexdef_worker(Oid indexrelid, int colno,
@@ -1271,7 +1370,6 @@ pg_get_indexdef_worker(Oid indexrelid, int colno,
 	return buf.data;
 }
 
-
 /*
  * pg_get_constraintdef
  *
@@ -1305,9 +1403,9 @@ pg_get_constraintdef_ext(PG_FUNCTION_ARGS)
 
 /* Internal version that returns a palloc'd C string; no pretty-printing */
 char *
-pg_get_constraintdef_string(Oid constraintId)
+pg_get_constraintdef_string(Oid constraintId, bool fullCommand)
 {
-	return pg_get_constraintdef_worker(constraintId, true, 0);
+	return pg_get_constraintdef_worker(constraintId, fullCommand, 0);
 }
 
 /*
@@ -2494,7 +2592,7 @@ deparse_expression(Node *expr, List *dpcontext,
  * The result is a palloc'd string.
  * ----------
  */
-static char *
+char *
 deparse_expression_pretty(Node *expr, List *dpcontext,
 						  bool forceprefix, bool showimplicit,
 						  int prettyFlags, int startIndent)
@@ -9165,7 +9263,7 @@ get_tablesample_def(TableSampleClause *tablesample, deparse_context *context)
  * actual_datatype.  (If you don't want this behavior, just pass
  * InvalidOid for actual_datatype.)
  */
-static void
+void
 get_opclass_name(Oid opclass, Oid actual_datatype,
 				 StringInfo buf)
 {
@@ -9713,7 +9811,7 @@ string_to_text(char *str)
 /*
  * Generate a C string representing a relation's reloptions, or NULL if none.
  */
-static char *
+char *
 flatten_reloptions(Oid relid)
 {
 	char	   *result = NULL;
diff --git a/src/include/catalog/dependency.h b/src/include/catalog/dependency.h
index fbcf904..b400095 100644
--- a/src/include/catalog/dependency.h
+++ b/src/include/catalog/dependency.h
@@ -143,6 +143,7 @@ typedef enum ObjectClass
 	OCLASS_TSDICT,				/* pg_ts_dict */
 	OCLASS_TSTEMPLATE,			/* pg_ts_template */
 	OCLASS_TSCONFIG,			/* pg_ts_config */
+	OCLASS_TSCONFIG_MAP,		/* pg_ts_config_map */
 	OCLASS_ROLE,				/* pg_authid */
 	OCLASS_DATABASE,			/* pg_database */
 	OCLASS_TBLSPACE,			/* pg_tablespace */
diff --git a/src/include/commands/sequence.h b/src/include/commands/sequence.h
index 44862bb..8d8e556 100644
--- a/src/include/commands/sequence.h
+++ b/src/include/commands/sequence.h
@@ -72,6 +72,7 @@ extern Datum setval3_oid(PG_FUNCTION_ARGS);
 extern Datum lastval(PG_FUNCTION_ARGS);
 
 extern Datum pg_sequence_parameters(PG_FUNCTION_ARGS);
+extern Form_pg_sequence get_sequence_values(Oid sequenceId);
 
 extern ObjectAddress DefineSequence(CreateSeqStmt *stmt);
 extern ObjectAddress AlterSequence(AlterSeqStmt *stmt);
diff --git a/src/include/nodes/parsenodes.h b/src/include/nodes/parsenodes.h
index 151c93a..a151412 100644
--- a/src/include/nodes/parsenodes.h
+++ b/src/include/nodes/parsenodes.h
@@ -1408,6 +1408,7 @@ typedef enum ObjectType
 	OBJECT_TRANSFORM,
 	OBJECT_TRIGGER,
 	OBJECT_TSCONFIGURATION,
+	OBJECT_TSCONFIG_MAPPING,
 	OBJECT_TSDICTIONARY,
 	OBJECT_TSPARSER,
 	OBJECT_TSTEMPLATE,
diff --git a/src/include/utils/builtins.h b/src/include/utils/builtins.h
index fc1679e..c3ff3d7 100644
--- a/src/include/utils/builtins.h
+++ b/src/include/utils/builtins.h
@@ -662,6 +662,7 @@ extern char *format_procedure(Oid procedure_oid);
 extern char *format_procedure_qualified(Oid procedure_oid);
 extern void format_procedure_parts(Oid operator_oid, List **objnames,
 					   List **objargs);
+extern char *format_procedure_args(Oid procedure_oid, bool force_qualify);
 extern char *format_operator(Oid operator_oid);
 extern char *format_operator_qualified(Oid operator_oid);
 extern void format_operator_parts(Oid operator_oid, List **objnames,
diff --git a/src/include/utils/ruleutils.h b/src/include/utils/ruleutils.h
index 3494b13..73b19d2 100644
--- a/src/include/utils/ruleutils.h
+++ b/src/include/utils/ruleutils.h
@@ -13,15 +13,25 @@
 #ifndef RULEUTILS_H
 #define RULEUTILS_H
 
+#include "catalog/pg_trigger.h"
 #include "nodes/nodes.h"
 #include "nodes/parsenodes.h"
 #include "nodes/pg_list.h"
 
 
+extern char *deparse_expression_pretty(Node *expr, List *dpcontext,
+						  bool forceprefix, bool showimplicit,
+						  int prettyFlags, int startIndent);
 extern char *pg_get_indexdef_string(Oid indexrelid);
 extern char *pg_get_indexdef_columns(Oid indexrelid, bool pretty);
+extern char *pg_get_trigger_whenclause(Form_pg_trigger trigrec,
+						  Node *whenClause, bool pretty);
+extern char *pg_get_constraintdef_string(Oid constraintId, bool fullCommand);
+extern void pg_get_ruledef_detailed(Datum ev_qual, Datum ev_action,
+					   char **whereClause, List **actions);
+extern char *pg_get_viewdef_internal(Oid viewoid);
+extern char *pg_get_createtableas_def(Query *query);
 
-extern char *pg_get_constraintdef_string(Oid constraintId);
 extern char *deparse_expression(Node *expr, List *dpcontext,
 				   bool forceprefix, bool showimplicit);
 extern List *deparse_context_for(const char *aliasname, Oid relid);
@@ -30,6 +40,9 @@ extern List *set_deparse_context_planstate(List *dpcontext,
 							  Node *planstate, List *ancestors);
 extern List *select_rtable_names_for_explain(List *rtable,
 								Bitmapset *rels_used);
+extern void get_opclass_name(Oid opclass, Oid actual_datatype,
+				 StringInfo buf);
 extern char *generate_collation_name(Oid collid);
+extern char *flatten_reloptions(Oid relid);
 
 #endif   /* RULEUTILS_H */
diff --git a/src/test/regress/GNUmakefile b/src/test/regress/GNUmakefile
index 4d6849e..1bcad70 100644
--- a/src/test/regress/GNUmakefile
+++ b/src/test/regress/GNUmakefile
@@ -123,6 +123,16 @@ tablespace-setup:
 	rm -rf ./testtablespace
 	mkdir ./testtablespace
 
+# DDL-deparse setup
+
+.PHONY: generate-files
+generate-files:
+	$(top_builddir)/src/test/regress/pg_regress --generate-files-only --inputdir=$(srcdir)/input
+
+ddl_deparse_schedule: serial_schedule
+	echo "test: deparse_init" > $@
+	grep -v tablespace $(srcdir)/serial_schedule >> $@
+	echo "test: deparse_test" >> $@
 
 ##
 ## Run tests
@@ -148,6 +158,18 @@ installcheck-tests: all tablespace-setup
 standbycheck: all
 	$(pg_regress_installcheck) $(REGRESS_OPTS) --schedule=$(srcdir)/standby_schedule --use-existing
 
+deparsecheck: REGRESS_OPTS += --keep-instance --load-extension=ddl_deparse
+deparsecheck: all temp-install generate-files ddl_deparse_schedule
+	$(MAKE) -C $(top_builddir)/contrib/ddl_deparse DESTDIR=$(abs_top_builddir)/tmp_install install
+	$(pg_regress_check) $(REGRESS_OPTS) --schedule=ddl_deparse_schedule
+	grep ERROR results/deparse_dump.out > results/deparse_dump.errors
+	$(abs_top_builddir)/tmp_install/$(bindir)/pg_ctl -w start -D ./tmp_check/data -l ./tmp_check/log/secondary.log -o "-p 49152"
+	$(abs_top_builddir)/tmp_install/$(bindir)/pg_dump -p 49152 -d regression_deparse -s -f results/deparse.dump
+	$(abs_top_builddir)/tmp_install/$(bindir)/pg_dump -p 49152 -d regression -s -f results/regression.dump
+	$(abs_top_builddir)/tmp_install/$(bindir)/pg_ctl -D ./tmp_check/data stop
+	diff -c results/deparse.dump results/regression.dump
+	diff -c results/deparse_dump.errors $(srcdir)/expected/deparse_dump.errors
+
 # old interfaces follow...
 
 runcheck: check
diff --git a/src/test/regress/expected/alter_table.out b/src/test/regress/expected/alter_table.out
index 28422ea..24d2675 100644
--- a/src/test/regress/expected/alter_table.out
+++ b/src/test/regress/expected/alter_table.out
@@ -1862,7 +1862,9 @@ where virtualtransaction = (
         from pg_locks
         where transactionid = txid_current()::integer)
 and locktype = 'relation'
-and relnamespace != (select oid from pg_namespace where nspname = 'pg_catalog')
+and relnamespace NOT IN (
+	select oid from pg_namespace
+	where nspname IN ('pg_catalog', 'pg_deparse'))
 and c.relname != 'my_locks'
 group by c.relname;
 create table alterlock (f1 int primary key, f2 text);
diff --git a/src/test/regress/expected/create_function_ddl_demo.out b/src/test/regress/expected/create_function_ddl_demo.out
new file mode 100644
index 0000000..ff8a1d9
--- /dev/null
+++ b/src/test/regress/expected/create_function_ddl_demo.out
@@ -0,0 +1,4 @@
+CREATE FUNCTION check_foreign_key ()
+	RETURNS trigger
+	AS '/space/sda1/ibarwick/2ndquadrant_bdr/src/test/regress/refint.so'
+	LANGUAGE C;
diff --git a/src/test/regress/expected/deparse_init.out b/src/test/regress/expected/deparse_init.out
new file mode 100644
index 0000000..f288be1
--- /dev/null
+++ b/src/test/regress/expected/deparse_init.out
@@ -0,0 +1,178 @@
+--
+-- DEPARSE_INIT
+--
+CREATE SCHEMA deparse;
+UPDATE pg_namespace SET nspname = 'pg_deparse' WHERE nspname = 'deparse';
+CREATE UNLOGGED TABLE pg_deparse.deparse_test_commands (
+  backend_id int,
+  backend_start timestamptz,
+  lsn pg_lsn,
+  ord integer,
+  command TEXT
+);
+CREATE OR REPLACE FUNCTION pg_deparse.deparse_test_ddl_command_end()
+  RETURNS event_trigger
+  SECURITY DEFINER
+  LANGUAGE plpgsql
+AS $fn$
+BEGIN
+	BEGIN
+		INSERT INTO pg_deparse.deparse_test_commands
+		            (backend_id, backend_start, command, ord, lsn)
+		SELECT id, pg_stat_get_backend_start(id),
+		     public.ddl_deparse_expand_command(public.ddl_deparse_to_json(command)),
+                     ordinality, lsn
+		FROM pg_event_trigger_dll_commands() WITH ORDINALITY,
+		pg_current_xlog_insert_location() lsn,
+		pg_stat_get_backend_idset() id
+		 WHERE pg_stat_get_backend_pid(id) = pg_backend_pid() AND
+		NOT command_tag = 'CREATE TABLE AS EXECUTE';
+	EXCEPTION WHEN OTHERS THEN 
+			RAISE WARNING 'state: % errm: %', sqlstate, sqlerrm;
+	END;
+END;
+$fn$;
+CREATE OR REPLACE FUNCTION pg_deparse.deparse_test_sql_drop()
+  RETURNS event_trigger
+  SECURITY DEFINER
+  LANGUAGE plpgsql
+AS $fn$
+DECLARE
+fmt	TEXT;
+obj RECORD;
+i	integer = 1;
+BEGIN
+
+	/* This function runs in the sql_drop event trigger.
+     *
+	 * When it runs, we know that all objects reported by the
+	 * pg_event_trigger_dropped_objects() function marked as "original" have
+	 * been mentioned in the DROP command, either directly by name or
+	 * indirectly by owner (DROP OWNED BY).  Since no objects that depend on
+	 * them can persist after that, we can replicate the effect of that by
+	 * executing an equivalent "DROP IF EXISTS object ... CASCADE".  CASCADE
+	 * lets the deletion work even in presence of objects that appear further
+	 * down in the return set of pg_event_trigger_dropped_objects, while IF
+	 * EXISTS let the deletion silently do nothing if the object was already
+	 * dropped because it was dependent on another object before it in the same
+	 * result set.
+     *
+     * (In general, it is impossible to reorder the result set in a way that
+     * would be completely free of dependency issues.)
+     */
+
+	FOR obj IN
+	SELECT object_type, address_names, address_args, object_identity
+	  FROM pg_event_trigger_dropped_objects()
+	 WHERE original
+	  LOOP
+
+		-- special case for default acls: ignore them.
+		IF obj.object_type = 'default acl' THEN
+			CONTINUE;
+		END IF;
+
+		/*
+		 * special cases for objects that are part of other objects: drop
+		 * each in a separate command.  Since we only deal with "original"
+		 * objects, these would not be reported in the complex case of
+		 * DROP OWNED.
+		 */
+		IF obj.object_type = 'table column' OR obj.object_type = 'foreign table column' THEN
+			fmt = format('ALTER TABLE %I.%I DROP COLUMN %I CASCADE',
+				obj.address_names[1],
+				obj.address_names[2],
+				obj.address_names[3]);
+			-- ignore these; they are output by ALTER TABLE itself
+			fmt := NULL;
+		ELSIF obj.object_type = 'composite type column' THEN
+			fmt = format('ALTER TYPE %I.%I DROP ATTRIBUTE %I CASCADE',
+				obj.address_names[1],
+				obj.address_names[2],
+				obj.address_names[3]);
+			-- ignore these; they are output by ALTER TYPE itself
+			fmt := NULL;
+		ELSIF obj.object_type = 'table constraint' THEN
+			fmt = format('ALTER TABLE %I.%I DROP CONSTRAINT %I CASCADE',
+				obj.address_names[1],
+				obj.address_names[2],
+				obj.address_names[3]);
+			-- ignore these; they are output by ALTER TABLE itself
+			fmt := NULL;
+		ELSIF obj.object_type = 'domain constraint' THEN
+			fmt = format('ALTER DOMAIN %s DROP CONSTRAINT %I CASCADE',
+				obj.address_names[1],
+				obj.address_args[1]);
+		ELSIF obj.object_type = 'default value' THEN
+			fmt = format('ALTER TABLE %I.%I ALTER COLUMN %I DROP DEFAULT',
+				obj.address_names[1],
+				obj.address_names[2],
+				obj.address_names[3]);
+		ELSIF obj.object_type = 'foreign-data wrapper' THEN
+			fmt = format('DROP FOREIGN DATA WRAPPER IF EXISTS %s CASCADE',
+				obj.object_identity);
+		ELSIF obj.object_type = 'user mapping' THEN
+			fmt = format('DROP USER MAPPING FOR %I SERVER %I',
+				obj.address_names[1], obj.address_args[1]);
+		ELSIF obj.object_type = 'operator of access method' THEN
+			fmt = format('ALTER OPERATOR FAMILY %I.%I USING %I DROP OPERATOR %s (%s, %s)',
+				obj.address_names[2], obj.address_names[3], obj.address_names[1], obj.address_names[4],
+				obj.address_args[1], obj.address_args[2]);
+			-- ignore these; they are output by ALTER OPERATOR FAMILY itself
+			fmt := NULL;
+		ELSIF obj.object_type = 'function of access method' THEN
+			fmt = format('ALTER OPERATOR FAMILY %I.%I USING %I DROP FUNCTION %s (%s, %s)',
+				obj.address_names[2], obj.address_names[3], obj.address_names[1], obj.address_names[4],
+				obj.address_args[1], obj.address_args[2]);
+			-- ignore these; they are output by ALTER OPERATOR FAMILY itself
+			fmt := NULL;
+		ELSE
+			-- all other cases
+			fmt := format('DROP %s IF EXISTS %s CASCADE',
+				obj.object_type, obj.object_identity);
+		END IF;
+
+		IF fmt IS NULL THEN
+			CONTINUE;
+		END IF;
+
+		fmt := fmt || ' /* DROP support */';
+
+		INSERT INTO pg_deparse.deparse_test_commands
+		            (backend_id, backend_start, lsn, ord, command)
+			 SELECT id, pg_stat_get_backend_start(id),
+			        pg_current_xlog_insert_location(), i, fmt
+			   FROM pg_stat_get_backend_idset() id
+			  WHERE pg_stat_get_backend_pid(id) = pg_backend_pid();
+		i := i + 1;
+	END LOOP;
+END;
+$fn$;
+CREATE OR REPLACE FUNCTION pg_deparse.output_commands() RETURNS SETOF text LANGUAGE PLPGSQL AS $$
+DECLARE
+        cmd text;
+        prev_id int = -1;
+        prev_start timestamptz = '-infinity';
+        sess_id int;
+        sess_start timestamptz;
+BEGIN
+   FOR cmd, sess_id, sess_start IN
+			   SELECT command, backend_id, backend_start
+                 FROM pg_deparse.deparse_test_commands
+			 ORDER BY lsn, ord
+   LOOP
+          IF (sess_id, sess_start) <> (prev_id, prev_start) THEN
+                prev_id := sess_id;
+                prev_start := sess_start;
+                RETURN NEXT '\c';
+          END IF;
+      RETURN NEXT cmd || ';' ;
+   END LOOP;
+END;
+$$;
+CREATE EVENT TRIGGER deparse_test_trg_sql_drop
+  ON sql_drop
+  EXECUTE PROCEDURE pg_deparse.deparse_test_sql_drop();
+CREATE EVENT TRIGGER deparse_test_trg_ddl_command_end
+  ON ddl_command_end
+  EXECUTE PROCEDURE pg_deparse.deparse_test_ddl_command_end();
diff --git a/src/test/regress/expected/object_address.out b/src/test/regress/expected/object_address.out
index 034c0b1..0da2a34 100644
--- a/src/test/regress/expected/object_address.out
+++ b/src/test/regress/expected/object_address.out
@@ -12,6 +12,7 @@ CREATE FOREIGN DATA WRAPPER addr_fdw;
 CREATE SERVER addr_fserv FOREIGN DATA WRAPPER addr_fdw;
 CREATE TEXT SEARCH DICTIONARY addr_ts_dict (template=simple);
 CREATE TEXT SEARCH CONFIGURATION addr_ts_conf (copy=english);
+ALTER TEXT SEARCH CONFIGURATION addr_ts_conf ALTER MAPPING FOR asciiword WITH addr_ts_dict;
 CREATE TEXT SEARCH TEMPLATE addr_ts_temp (lexize=dsimple_lexize);
 CREATE TEXT SEARCH PARSER addr_ts_prs
     (start = prsd_start, gettoken = prsd_nexttoken, end = prsd_end, lextypes = prsd_lextype);
@@ -79,6 +80,7 @@ BEGIN
 		('operator'), ('operator class'), ('operator family'), ('rule'), ('trigger'),
 		('text search parser'), ('text search dictionary'),
 		('text search template'), ('text search configuration'),
+		('text search configuration mapping'),
 		('policy'), ('user mapping'), ('default acl'), ('transform'),
 		('operator of access method'), ('function of access method')
 	LOOP
@@ -246,6 +248,12 @@ WARNING:  error for text search configuration,{addr_nsp,zwei},{}: text search co
 WARNING:  error for text search configuration,{addr_nsp,zwei},{integer}: text search configuration "addr_nsp.zwei" does not exist
 WARNING:  error for text search configuration,{eins,zwei,drei},{}: cross-database references are not implemented: eins.zwei.drei
 WARNING:  error for text search configuration,{eins,zwei,drei},{integer}: cross-database references are not implemented: eins.zwei.drei
+WARNING:  error for text search configuration mapping,{eins},{}: text search configuration "eins" does not exist
+WARNING:  error for text search configuration mapping,{eins},{integer}: text search configuration "eins" does not exist
+WARNING:  error for text search configuration mapping,{addr_nsp,zwei},{}: text search configuration "addr_nsp.zwei" does not exist
+WARNING:  error for text search configuration mapping,{addr_nsp,zwei},{integer}: text search configuration "addr_nsp.zwei" does not exist
+WARNING:  error for text search configuration mapping,{eins,zwei,drei},{}: cross-database references are not implemented: eins.zwei.drei
+WARNING:  error for text search configuration mapping,{eins,zwei,drei},{integer}: cross-database references are not implemented: eins.zwei.drei
 WARNING:  error for policy,{eins},{}: must specify relation and object name
 WARNING:  error for policy,{eins},{integer}: must specify relation and object name
 WARNING:  error for policy,{addr_nsp,zwei},{}: relation "addr_nsp" does not exist
@@ -362,6 +370,7 @@ WITH objects (type, name, args) AS (VALUES
 				('text search dictionary', '{addr_ts_dict}', '{}'),
 				('text search template', '{addr_ts_temp}', '{}'),
 				('text search configuration', '{addr_ts_conf}', '{}'),
+				('text search configuration mapping', '{addr_ts_conf}', '{}'),
 				('role', '{regtest_addr_user}', '{}'),
 				-- database
 				-- tablespace
@@ -383,50 +392,51 @@ SELECT (pg_identify_object(addr1.classid, addr1.objid, addr1.subobjid)).*,
 			pg_identify_object_as_address(classid, objid, subobjid) ioa(typ,nms,args),
 			pg_get_object_address(typ, nms, ioa.args) as addr2
 	ORDER BY addr1.classid, addr1.objid, addr1.subobjid;
-           type            |   schema   |       name        |                               identity                               | ?column? 
----------------------------+------------+-------------------+----------------------------------------------------------------------+----------
- default acl               |            |                   | for role regtest_addr_user in schema public on tables                | t
- default acl               |            |                   | for role regtest_addr_user on tables                                 | t
- type                      | pg_catalog | _int4             | integer[]                                                            | t
- type                      | addr_nsp   | gencomptype       | addr_nsp.gencomptype                                                 | t
- type                      | addr_nsp   | genenum           | addr_nsp.genenum                                                     | t
- type                      | addr_nsp   | gendomain         | addr_nsp.gendomain                                                   | t
- function                  | pg_catalog |                   | pg_catalog.pg_identify_object(pg_catalog.oid,pg_catalog.oid,integer) | t
- aggregate                 | addr_nsp   |                   | addr_nsp.genaggr(integer)                                            | t
- sequence                  | addr_nsp   | gentable_a_seq    | addr_nsp.gentable_a_seq                                              | t
- table                     | addr_nsp   | gentable          | addr_nsp.gentable                                                    | t
- table column              | addr_nsp   | gentable          | addr_nsp.gentable.b                                                  | t
- index                     | addr_nsp   | gentable_pkey     | addr_nsp.gentable_pkey                                               | t
- view                      | addr_nsp   | genview           | addr_nsp.genview                                                     | t
- materialized view         | addr_nsp   | genmatview        | addr_nsp.genmatview                                                  | t
- foreign table             | addr_nsp   | genftable         | addr_nsp.genftable                                                   | t
- foreign table column      | addr_nsp   | genftable         | addr_nsp.genftable.a                                                 | t
- role                      |            | regtest_addr_user | regtest_addr_user                                                    | t
- server                    |            | addr_fserv        | addr_fserv                                                           | t
- user mapping              |            |                   | regtest_addr_user on server integer                                  | t
- foreign-data wrapper      |            | addr_fdw          | addr_fdw                                                             | t
- operator of access method |            |                   | operator 1 (integer, integer) of pg_catalog.integer_ops USING btree  | t
- function of access method |            |                   | function 2 (integer, integer) of pg_catalog.integer_ops USING btree  | t
- default value             |            |                   | for addr_nsp.gentable.b                                              | t
- cast                      |            |                   | (bigint AS integer)                                                  | t
- table constraint          | addr_nsp   |                   | a_chk on addr_nsp.gentable                                           | t
- domain constraint         | addr_nsp   |                   | domconstr on addr_nsp.gendomain                                      | t
- conversion                | pg_catalog | ascii_to_mic      | pg_catalog.ascii_to_mic                                              | t
- language                  |            | plpgsql           | plpgsql                                                              | t
- schema                    |            | addr_nsp          | addr_nsp                                                             | t
- operator class            | pg_catalog | int4_ops          | pg_catalog.int4_ops USING btree                                      | t
- operator                  | pg_catalog |                   | pg_catalog.+(integer,integer)                                        | t
- rule                      |            |                   | "_RETURN" on addr_nsp.genview                                        | t
- trigger                   |            |                   | t on addr_nsp.gentable                                               | t
- operator family           | pg_catalog | integer_ops       | pg_catalog.integer_ops USING btree                                   | t
- policy                    |            |                   | genpol on addr_nsp.gentable                                          | t
- collation                 | pg_catalog | "default"         | pg_catalog."default"                                                 | t
- transform                 |            |                   | for integer on language sql                                          | t
- text search dictionary    | addr_nsp   | addr_ts_dict      | addr_nsp.addr_ts_dict                                                | t
- text search parser        | addr_nsp   | addr_ts_prs       | addr_nsp.addr_ts_prs                                                 | t
- text search configuration | addr_nsp   | addr_ts_conf      | addr_nsp.addr_ts_conf                                                | t
- text search template      | addr_nsp   | addr_ts_temp      | addr_nsp.addr_ts_temp                                                | t
-(41 rows)
+               type                |   schema   |       name        |                               identity                               | ?column? 
+-----------------------------------+------------+-------------------+----------------------------------------------------------------------+----------
+ default acl                       |            |                   | for role regtest_addr_user in schema public on tables                | t
+ default acl                       |            |                   | for role regtest_addr_user on tables                                 | t
+ type                              | pg_catalog | _int4             | integer[]                                                            | t
+ type                              | addr_nsp   | gencomptype       | addr_nsp.gencomptype                                                 | t
+ type                              | addr_nsp   | genenum           | addr_nsp.genenum                                                     | t
+ type                              | addr_nsp   | gendomain         | addr_nsp.gendomain                                                   | t
+ function                          | pg_catalog |                   | pg_catalog.pg_identify_object(pg_catalog.oid,pg_catalog.oid,integer) | t
+ aggregate                         | addr_nsp   |                   | addr_nsp.genaggr(integer)                                            | t
+ sequence                          | addr_nsp   | gentable_a_seq    | addr_nsp.gentable_a_seq                                              | t
+ table                             | addr_nsp   | gentable          | addr_nsp.gentable                                                    | t
+ table column                      | addr_nsp   | gentable          | addr_nsp.gentable.b                                                  | t
+ index                             | addr_nsp   | gentable_pkey     | addr_nsp.gentable_pkey                                               | t
+ view                              | addr_nsp   | genview           | addr_nsp.genview                                                     | t
+ materialized view                 | addr_nsp   | genmatview        | addr_nsp.genmatview                                                  | t
+ foreign table                     | addr_nsp   | genftable         | addr_nsp.genftable                                                   | t
+ foreign table column              | addr_nsp   | genftable         | addr_nsp.genftable.a                                                 | t
+ role                              |            | regtest_addr_user | regtest_addr_user                                                    | t
+ server                            |            | addr_fserv        | addr_fserv                                                           | t
+ user mapping                      |            |                   | regtest_addr_user on server integer                                  | t
+ foreign-data wrapper              |            | addr_fdw          | addr_fdw                                                             | t
+ operator of access method         |            |                   | operator 1 (integer, integer) of pg_catalog.integer_ops USING btree  | t
+ function of access method         |            |                   | function 2 (integer, integer) of pg_catalog.integer_ops USING btree  | t
+ default value                     |            |                   | for addr_nsp.gentable.b                                              | t
+ cast                              |            |                   | (bigint AS integer)                                                  | t
+ table constraint                  | addr_nsp   |                   | a_chk on addr_nsp.gentable                                           | t
+ domain constraint                 | addr_nsp   |                   | domconstr on addr_nsp.gendomain                                      | t
+ conversion                        | pg_catalog | ascii_to_mic      | pg_catalog.ascii_to_mic                                              | t
+ language                          |            | plpgsql           | plpgsql                                                              | t
+ schema                            |            | addr_nsp          | addr_nsp                                                             | t
+ operator class                    | pg_catalog | int4_ops          | pg_catalog.int4_ops USING btree                                      | t
+ operator                          | pg_catalog |                   | pg_catalog.+(integer,integer)                                        | t
+ rule                              |            |                   | "_RETURN" on addr_nsp.genview                                        | t
+ trigger                           |            |                   | t on addr_nsp.gentable                                               | t
+ operator family                   | pg_catalog | integer_ops       | pg_catalog.integer_ops USING btree                                   | t
+ policy                            |            |                   | genpol on addr_nsp.gentable                                          | t
+ collation                         | pg_catalog | "default"         | pg_catalog."default"                                                 | t
+ transform                         |            |                   | for integer on language sql                                          | t
+ text search dictionary            | addr_nsp   | addr_ts_dict      | addr_nsp.addr_ts_dict                                                | t
+ text search parser                | addr_nsp   | addr_ts_prs       | addr_nsp.addr_ts_prs                                                 | t
+ text search configuration         | addr_nsp   | addr_ts_conf      | addr_nsp.addr_ts_conf                                                | t
+ text search configuration mapping |            |                   | addr_nsp.addr_ts_conf                                                | t
+ text search template              | addr_nsp   | addr_ts_temp      | addr_nsp.addr_ts_temp                                                | t
+(42 rows)
 
 ---
 --- Cleanup resources
diff --git a/src/test/regress/expected/sanity_check.out b/src/test/regress/expected/sanity_check.out
index eb0bc88..ab9af544 100644
--- a/src/test/regress/expected/sanity_check.out
+++ b/src/test/regress/expected/sanity_check.out
@@ -3,13 +3,14 @@ VACUUM;
 -- sanity check, if we don't have indices the test will take years to
 -- complete.  But skip TOAST relations (since they will have varying
 -- names depending on the current OID counter) as well as temp tables
--- of other backends (to avoid timing-dependent behavior).
+-- of other backends (to avoid timing-dependent behavior).  Also exclude
+-- the schema used for the deparse test, as it might not be there at all.
 --
 -- temporarily disable fancy output, so catalog changes create less diff noise
 \a\t
 SELECT relname, relhasindex
    FROM pg_class c LEFT JOIN pg_namespace n ON n.oid = relnamespace
-   WHERE relkind = 'r' AND (nspname ~ '^pg_temp_') IS NOT TRUE
+   WHERE relkind = 'r' AND (nspname ~ '^pg_temp_' OR nspname ~ '^pg_deparse') IS NOT TRUE
    ORDER BY relname;
 a|f
 a_star|f
diff --git a/src/test/regress/input/deparse_test.source b/src/test/regress/input/deparse_test.source
new file mode 100644
index 0000000..e7ba0dd
--- /dev/null
+++ b/src/test/regress/input/deparse_test.source
@@ -0,0 +1,57 @@
+---
+--- DEPARSE_TEST
+---
+
+-- create roles used throughout the tests
+create role clstr_user;
+create role "current_user";
+create role foreign_data_user;
+create role "Public";
+create role regressgroup1;
+create role regressgroup2;
+create role regression_bob;
+create role regression_group;
+create role regression_user1;
+create role regression_user2;
+create role regression_user3;
+create role regression_user;
+create role regresslo;
+create role regress_rol_lock1;
+create role regress_test_indirect;
+create role regress_test_role;
+create role regress_test_role2;
+create role regress_test_role_super superuser;
+create role regressuser1;
+create role regressuser2;
+create role regressuser3;
+create role regressuser4;
+create role regressuser5;
+create role regtest_unpriv_user;
+create role regtest_addr_user;
+create role regtest_alter_user1;
+create role regtest_alter_user2;
+create role regtest_alter_user3;
+create role rls_regress_group1;
+create role rls_regress_group2;
+create role rls_regress_user0;
+create role rls_regress_user1;
+create role rls_regress_user2;
+create role rls_regress_exempt_user;
+create role schemauser2;
+create role seclabel_user1;
+create role seclabel_user2;
+create role selinto_user;
+create role testrol1;
+create role testrol2;
+create role testrolx;
+create role unprivileged_role;
+create role "user";
+create role view_user2;
+
+\pset format unaligned
+\pset tuples_only
+\o ./sql/deparse_dump.sql
+
+SELECT * FROM pg_deparse.output_commands();
+
+\! @abs_builddir@/../../bin/psql/psql --dbname=@deparse_test_db@ -e < ./sql/deparse_dump.sql > results/deparse_dump.out 2>&1
diff --git a/src/test/regress/output/deparse_test.source b/src/test/regress/output/deparse_test.source
new file mode 100644
index 0000000..f083e7c
--- /dev/null
+++ b/src/test/regress/output/deparse_test.source
@@ -0,0 +1,53 @@
+---
+--- DEPARSE_TEST
+---
+-- create roles used throughout the tests
+create role clstr_user;
+create role "current_user";
+create role foreign_data_user;
+create role "Public";
+create role regressgroup1;
+create role regressgroup2;
+create role regression_bob;
+create role regression_group;
+create role regression_user1;
+create role regression_user2;
+create role regression_user3;
+create role regression_user;
+create role regresslo;
+create role regress_rol_lock1;
+create role regress_test_indirect;
+create role regress_test_role;
+create role regress_test_role2;
+create role regress_test_role_super superuser;
+create role regressuser1;
+create role regressuser2;
+create role regressuser3;
+create role regressuser4;
+create role regressuser5;
+create role regtest_unpriv_user;
+create role regtest_addr_user;
+create role regtest_alter_user1;
+create role regtest_alter_user2;
+create role regtest_alter_user3;
+create role rls_regress_group1;
+create role rls_regress_group2;
+create role rls_regress_user0;
+create role rls_regress_user1;
+create role rls_regress_user2;
+create role rls_regress_exempt_user;
+create role schemauser2;
+create role seclabel_user1;
+create role seclabel_user2;
+create role selinto_user;
+create role testrol1;
+create role testrol2;
+create role testrolx;
+create role unprivileged_role;
+create role "user";
+create role view_user2;
+\pset format unaligned
+\pset tuples_only
+\o ./sql/deparse_dump.sql
+SELECT * FROM pg_deparse.output_commands();
+\! @abs_builddir@/../../bin/psql/psql --dbname=@deparse_test_db@ -e < ./sql/deparse_dump.sql > results/deparse_dump.out 2>&1
diff --git a/src/test/regress/pg_regress.c b/src/test/regress/pg_regress.c
index dd65ab5..fe00929 100644
--- a/src/test/regress/pg_regress.c
+++ b/src/test/regress/pg_regress.c
@@ -68,6 +68,7 @@ const char *pretty_diff_opts = "-w -C3";
 
 /* options settable from command line */
 _stringlist *dblist = NULL;
+char	    deparse_test_db[NAMEDATALEN];
 bool		debug = false;
 char	   *inputdir = ".";
 char	   *outputdir = ".";
@@ -90,6 +91,8 @@ static char *dlpath = PKGLIBDIR;
 static char *user = NULL;
 static _stringlist *extraroles = NULL;
 static char *config_auth_datadir = NULL;
+static bool generate_files_only = false;
+static bool keep_instance = false;
 
 /* internal variables */
 static const char *progname;
@@ -554,6 +557,7 @@ convert_sourcefiles_in(char *source_subdir, char *dest_dir, char *dest_subdir, c
 			replace_string(line, "@testtablespace@", testtablespace);
 			replace_string(line, "@libdir@", dlpath);
 			replace_string(line, "@DLSUFFIX@", DLSUFFIX);
+			replace_string(line, "@deparse_test_db@", deparse_test_db);
 			fputs(line, outfile);
 		}
 		fclose(infile);
@@ -1958,6 +1962,8 @@ help(void)
 	printf(_("  --config-auth=DATADIR     update authentication settings for DATADIR\n"));
 	printf(_("  --create-role=ROLE        create the specified role before testing\n"));
 	printf(_("  --dbname=DB               use database DB (default \"regression\")\n"));
+	printf(_("  --dbname-deparse=DB       use database DB for DDL deparse test (default\n"));
+	printf(_("                            \"regression_deparse\")\n"));
 	printf(_("  --debug                   turn on debug mode in programs that are run\n"));
 	printf(_("  --dlpath=DIR              look for dynamic libraries in DIR\n"));
 	printf(_("  --encoding=ENCODING       use ENCODING as the encoding\n"));
@@ -1974,6 +1980,7 @@ help(void)
 	printf(_("                            (can be used multiple times to concatenate)\n"));
 	printf(_("  --temp-instance=DIR       create a temporary instance in DIR\n"));
 	printf(_("  --use-existing            use an existing installation\n"));
+	printf(_("  --keep-instance           don't destroy nor stop the temp instance\n"));
 	printf(_("\n"));
 	printf(_("Options for \"temp-instance\" mode:\n"));
 	printf(_("  --no-locale               use C locale\n"));
@@ -2018,6 +2025,9 @@ regression_main(int argc, char *argv[], init_function ifunc, test_function tfunc
 		{"launcher", required_argument, NULL, 21},
 		{"load-extension", required_argument, NULL, 22},
 		{"config-auth", required_argument, NULL, 24},
+		{"dbname-deparse", required_argument, NULL, 25},
+		{"generate-files-only", no_argument, NULL, 26},
+		{"keep-instance", no_argument, NULL, 27},
 		{NULL, 0, NULL, 0}
 	};
 
@@ -2065,6 +2075,14 @@ regression_main(int argc, char *argv[], init_function ifunc, test_function tfunc
 				 */
 				free_stringlist(&dblist);
 				split_to_stringlist(strdup(optarg), ", ", &dblist);
+
+				/*
+				 * We need to update the default deparse test db name
+				 * to be a suffix of the provided database name
+				 */
+				snprintf(deparse_test_db, sizeof(deparse_test_db),
+						 "%s_deparse",
+						 dblist->str);
 				break;
 			case 2:
 				debug = true;
@@ -2131,6 +2149,15 @@ regression_main(int argc, char *argv[], init_function ifunc, test_function tfunc
 			case 24:
 				config_auth_datadir = pstrdup(optarg);
 				break;
+			case 25:
+				strncpy(deparse_test_db, optarg, sizeof(deparse_test_db));
+				break;
+			case 26:
+				generate_files_only = true;
+				break;
+			case 27:
+				keep_instance = true;
+				break;
 			default:
 				/* getopt_long already emitted a complaint */
 				fprintf(stderr, _("\nTry \"%s -h\" for more information.\n"),
@@ -2171,6 +2198,12 @@ regression_main(int argc, char *argv[], init_function ifunc, test_function tfunc
 	outputdir = make_absolute_path(outputdir);
 	dlpath = make_absolute_path(dlpath);
 
+	if (generate_files_only)
+	{
+		convert_sourcefiles();
+		exit(0);
+	}
+
 	/*
 	 * Initialization
 	 */
@@ -2404,6 +2437,7 @@ regression_main(int argc, char *argv[], init_function ifunc, test_function tfunc
 		{
 			for (sl = dblist; sl; sl = sl->next)
 				drop_database_if_exists(sl->str);
+			drop_database_if_exists(deparse_test_db);
 			for (sl = extraroles; sl; sl = sl->next)
 				drop_role_if_exists(sl->str);
 		}
@@ -2416,6 +2450,7 @@ regression_main(int argc, char *argv[], init_function ifunc, test_function tfunc
 	{
 		for (sl = dblist; sl; sl = sl->next)
 			create_database(sl->str);
+		create_database(deparse_test_db);
 		for (sl = extraroles; sl; sl = sl->next)
 			create_role(sl->str, dblist);
 	}
@@ -2438,7 +2473,7 @@ regression_main(int argc, char *argv[], init_function ifunc, test_function tfunc
 	/*
 	 * Shut down temp installation's postmaster
 	 */
-	if (temp_instance)
+	if (temp_instance && !keep_instance)
 	{
 		header(_("shutting down postmaster"));
 		stop_postmaster();
@@ -2449,7 +2484,7 @@ regression_main(int argc, char *argv[], init_function ifunc, test_function tfunc
 	 * conserve disk space.  (If there were errors, we leave the instance in
 	 * place for possible manual investigation.)
 	 */
-	if (temp_instance && fail_count == 0 && fail_ignore_count == 0)
+	if (temp_instance && fail_count == 0 && fail_ignore_count == 0 && !keep_instance)
 	{
 		header(_("removing temporary instance"));
 		if (!rmtree(temp_instance, true))
diff --git a/src/test/regress/pg_regress.h b/src/test/regress/pg_regress.h
index 0fc9db7..8411d4f 100644
--- a/src/test/regress/pg_regress.h
+++ b/src/test/regress/pg_regress.h
@@ -38,6 +38,7 @@ extern char *datadir;
 extern char *host_platform;
 
 extern _stringlist *dblist;
+extern char deparse_test_db[NAMEDATALEN];
 extern bool debug;
 extern char *inputdir;
 extern char *outputdir;
diff --git a/src/test/regress/pg_regress_main.c b/src/test/regress/pg_regress_main.c
index 860431b..dd78805 100644
--- a/src/test/regress/pg_regress_main.c
+++ b/src/test/regress/pg_regress_main.c
@@ -88,6 +88,7 @@ psql_init(int argc, char **argv)
 {
 	/* set default regression database name */
 	add_stringlist_item(&dblist, "regression");
+	strlcpy(deparse_test_db, "regression_deparse", NAMEDATALEN);
 }
 
 int
diff --git a/src/test/regress/sql/alter_table.sql b/src/test/regress/sql/alter_table.sql
index 3ef55d9..01726a1 100644
--- a/src/test/regress/sql/alter_table.sql
+++ b/src/test/regress/sql/alter_table.sql
@@ -1291,7 +1291,9 @@ where virtualtransaction = (
         from pg_locks
         where transactionid = txid_current()::integer)
 and locktype = 'relation'
-and relnamespace != (select oid from pg_namespace where nspname = 'pg_catalog')
+and relnamespace NOT IN (
+	select oid from pg_namespace
+	where nspname IN ('pg_catalog', 'pg_deparse'))
 and c.relname != 'my_locks'
 group by c.relname;
 
diff --git a/src/test/regress/sql/create_function_ddl_demo.sql b/src/test/regress/sql/create_function_ddl_demo.sql
new file mode 100644
index 0000000..ac29426
--- /dev/null
+++ b/src/test/regress/sql/create_function_ddl_demo.sql
@@ -0,0 +1,4 @@
+CREATE FUNCTION check_foreign_key ()
+	RETURNS trigger
+	AS '/space/sda1/ibarwick/2ndquadrant_bdr/src/test/regress/refint.so'
+	LANGUAGE C;
\ No newline at end of file
diff --git a/src/test/regress/sql/deparse_init.sql b/src/test/regress/sql/deparse_init.sql
new file mode 100644
index 0000000..996c914
--- /dev/null
+++ b/src/test/regress/sql/deparse_init.sql
@@ -0,0 +1,182 @@
+--
+-- DEPARSE_INIT
+--
+CREATE SCHEMA deparse;
+UPDATE pg_namespace SET nspname = 'pg_deparse' WHERE nspname = 'deparse';
+CREATE UNLOGGED TABLE pg_deparse.deparse_test_commands (
+  backend_id int,
+  backend_start timestamptz,
+  lsn pg_lsn,
+  ord integer,
+  command TEXT
+);
+CREATE OR REPLACE FUNCTION pg_deparse.deparse_test_ddl_command_end()
+  RETURNS event_trigger
+  SECURITY DEFINER
+  LANGUAGE plpgsql
+AS $fn$
+BEGIN
+	BEGIN
+		INSERT INTO pg_deparse.deparse_test_commands
+		            (backend_id, backend_start, command, ord, lsn)
+		SELECT id, pg_stat_get_backend_start(id),
+		     public.ddl_deparse_expand_command(public.ddl_deparse_to_json(command)),
+                     ordinality, lsn
+		FROM pg_event_trigger_ddl_commands() WITH ORDINALITY,
+		pg_current_xlog_insert_location() lsn,
+		pg_stat_get_backend_idset() id
+		 WHERE pg_stat_get_backend_pid(id) = pg_backend_pid() AND
+		NOT command_tag = 'CREATE TABLE AS EXECUTE';
+	EXCEPTION WHEN OTHERS THEN 
+			RAISE WARNING 'state: % errm: %', sqlstate, sqlerrm;
+	END;
+END;
+$fn$;
+
+CREATE OR REPLACE FUNCTION pg_deparse.deparse_test_sql_drop()
+  RETURNS event_trigger
+  SECURITY DEFINER
+  LANGUAGE plpgsql
+AS $fn$
+DECLARE
+fmt	TEXT;
+obj RECORD;
+i	integer = 1;
+BEGIN
+
+	/* This function runs in the sql_drop event trigger.
+     *
+	 * When it runs, we know that all objects reported by the
+	 * pg_event_trigger_dropped_objects() function marked as "original" have
+	 * been mentioned in the DROP command, either directly by name or
+	 * indirectly by owner (DROP OWNED BY).  Since no objects that depend on
+	 * them can persist after that, we can replicate the effect of that by
+	 * executing an equivalent "DROP IF EXISTS object ... CASCADE".  CASCADE
+	 * lets the deletion work even in presence of objects that appear further
+	 * down in the return set of pg_event_trigger_dropped_objects, while IF
+	 * EXISTS let the deletion silently do nothing if the object was already
+	 * dropped because it was dependent on another object before it in the same
+	 * result set.
+     *
+     * (In general, it is impossible to reorder the result set in a way that
+     * would be completely free of dependency issues.)
+     */
+
+	FOR obj IN
+	SELECT object_type, address_names, address_args, object_identity
+	  FROM pg_event_trigger_dropped_objects()
+	 WHERE original
+	  LOOP
+
+		-- special case for default acls: ignore them.
+		IF obj.object_type = 'default acl' THEN
+			CONTINUE;
+		END IF;
+
+		/*
+		 * special cases for objects that are part of other objects: drop
+		 * each in a separate command.  Since we only deal with "original"
+		 * objects, these would not be reported in the complex case of
+		 * DROP OWNED.
+		 */
+		IF obj.object_type = 'table column' OR obj.object_type = 'foreign table column' THEN
+			fmt = format('ALTER TABLE %I.%I DROP COLUMN %I CASCADE',
+				obj.address_names[1],
+				obj.address_names[2],
+				obj.address_names[3]);
+			-- ignore these; they are output by ALTER TABLE itself
+			fmt := NULL;
+		ELSIF obj.object_type = 'composite type column' THEN
+			fmt = format('ALTER TYPE %I.%I DROP ATTRIBUTE %I CASCADE',
+				obj.address_names[1],
+				obj.address_names[2],
+				obj.address_names[3]);
+			-- ignore these; they are output by ALTER TYPE itself
+			fmt := NULL;
+		ELSIF obj.object_type = 'table constraint' THEN
+			fmt = format('ALTER TABLE %I.%I DROP CONSTRAINT %I CASCADE',
+				obj.address_names[1],
+				obj.address_names[2],
+				obj.address_names[3]);
+			-- ignore these; they are output by ALTER TABLE itself
+			fmt := NULL;
+		ELSIF obj.object_type = 'domain constraint' THEN
+			fmt = format('ALTER DOMAIN %s DROP CONSTRAINT %I CASCADE',
+				obj.address_names[1],
+				obj.address_args[1]);
+		ELSIF obj.object_type = 'default value' THEN
+			fmt = format('ALTER TABLE %I.%I ALTER COLUMN %I DROP DEFAULT',
+				obj.address_names[1],
+				obj.address_names[2],
+				obj.address_names[3]);
+		ELSIF obj.object_type = 'foreign-data wrapper' THEN
+			fmt = format('DROP FOREIGN DATA WRAPPER IF EXISTS %s CASCADE',
+				obj.object_identity);
+		ELSIF obj.object_type = 'user mapping' THEN
+			fmt = format('DROP USER MAPPING FOR %I SERVER %I',
+				obj.address_names[1], obj.address_args[1]);
+		ELSIF obj.object_type = 'operator of access method' THEN
+			fmt = format('ALTER OPERATOR FAMILY %I.%I USING %I DROP OPERATOR %s (%s, %s)',
+				obj.address_names[2], obj.address_names[3], obj.address_names[1], obj.address_names[4],
+				obj.address_args[1], obj.address_args[2]);
+			-- ignore these; they are output by ALTER OPERATOR FAMILY itself
+			fmt := NULL;
+		ELSIF obj.object_type = 'function of access method' THEN
+			fmt = format('ALTER OPERATOR FAMILY %I.%I USING %I DROP FUNCTION %s (%s, %s)',
+				obj.address_names[2], obj.address_names[3], obj.address_names[1], obj.address_names[4],
+				obj.address_args[1], obj.address_args[2]);
+			-- ignore these; they are output by ALTER OPERATOR FAMILY itself
+			fmt := NULL;
+		ELSE
+			-- all other cases
+			fmt := format('DROP %s IF EXISTS %s CASCADE',
+				obj.object_type, obj.object_identity);
+		END IF;
+
+		IF fmt IS NULL THEN
+			CONTINUE;
+		END IF;
+
+		fmt := fmt || ' /* DROP support */';
+
+		INSERT INTO pg_deparse.deparse_test_commands
+		            (backend_id, backend_start, lsn, ord, command)
+			 SELECT id, pg_stat_get_backend_start(id),
+			        pg_current_xlog_insert_location(), i, fmt
+			   FROM pg_stat_get_backend_idset() id
+			  WHERE pg_stat_get_backend_pid(id) = pg_backend_pid();
+		i := i + 1;
+	END LOOP;
+END;
+$fn$;
+
+CREATE OR REPLACE FUNCTION pg_deparse.output_commands() RETURNS SETOF text LANGUAGE PLPGSQL AS $$
+DECLARE
+        cmd text;
+        prev_id int = -1;
+        prev_start timestamptz = '-infinity';
+        sess_id int;
+        sess_start timestamptz;
+BEGIN
+   FOR cmd, sess_id, sess_start IN
+			   SELECT command, backend_id, backend_start
+                 FROM pg_deparse.deparse_test_commands
+			 ORDER BY lsn, ord
+   LOOP
+          IF (sess_id, sess_start) <> (prev_id, prev_start) THEN
+                prev_id := sess_id;
+                prev_start := sess_start;
+                RETURN NEXT '\c';
+          END IF;
+      RETURN NEXT cmd || ';' ;
+   END LOOP;
+END;
+$$;
+
+CREATE EVENT TRIGGER deparse_test_trg_sql_drop
+  ON sql_drop
+  EXECUTE PROCEDURE pg_deparse.deparse_test_sql_drop();
+
+CREATE EVENT TRIGGER deparse_test_trg_ddl_command_end
+  ON ddl_command_end
+  EXECUTE PROCEDURE pg_deparse.deparse_test_ddl_command_end();
diff --git a/src/test/regress/sql/object_address.sql b/src/test/regress/sql/object_address.sql
index 68e7cb0..a4f0aee 100644
--- a/src/test/regress/sql/object_address.sql
+++ b/src/test/regress/sql/object_address.sql
@@ -16,6 +16,7 @@ CREATE FOREIGN DATA WRAPPER addr_fdw;
 CREATE SERVER addr_fserv FOREIGN DATA WRAPPER addr_fdw;
 CREATE TEXT SEARCH DICTIONARY addr_ts_dict (template=simple);
 CREATE TEXT SEARCH CONFIGURATION addr_ts_conf (copy=english);
+ALTER TEXT SEARCH CONFIGURATION addr_ts_conf ALTER MAPPING FOR asciiword WITH addr_ts_dict;
 CREATE TEXT SEARCH TEMPLATE addr_ts_temp (lexize=dsimple_lexize);
 CREATE TEXT SEARCH PARSER addr_ts_prs
     (start = prsd_start, gettoken = prsd_nexttoken, end = prsd_end, lextypes = prsd_lextype);
@@ -77,6 +78,7 @@ BEGIN
 		('operator'), ('operator class'), ('operator family'), ('rule'), ('trigger'),
 		('text search parser'), ('text search dictionary'),
 		('text search template'), ('text search configuration'),
+		('text search configuration mapping'),
 		('policy'), ('user mapping'), ('default acl'), ('transform'),
 		('operator of access method'), ('function of access method')
 	LOOP
@@ -155,6 +157,7 @@ WITH objects (type, name, args) AS (VALUES
 				('text search dictionary', '{addr_ts_dict}', '{}'),
 				('text search template', '{addr_ts_temp}', '{}'),
 				('text search configuration', '{addr_ts_conf}', '{}'),
+				('text search configuration mapping', '{addr_ts_conf}', '{}'),
 				('role', '{regtest_addr_user}', '{}'),
 				-- database
 				-- tablespace
diff --git a/src/test/regress/sql/sanity_check.sql b/src/test/regress/sql/sanity_check.sql
index 0da838e..1d7a276 100644
--- a/src/test/regress/sql/sanity_check.sql
+++ b/src/test/regress/sql/sanity_check.sql
@@ -4,7 +4,8 @@ VACUUM;
 -- sanity check, if we don't have indices the test will take years to
 -- complete.  But skip TOAST relations (since they will have varying
 -- names depending on the current OID counter) as well as temp tables
--- of other backends (to avoid timing-dependent behavior).
+-- of other backends (to avoid timing-dependent behavior).  Also exclude
+-- the schema used for the deparse test, as it might not be there at all.
 --
 
 -- temporarily disable fancy output, so catalog changes create less diff noise
@@ -12,7 +13,7 @@ VACUUM;
 
 SELECT relname, relhasindex
    FROM pg_class c LEFT JOIN pg_namespace n ON n.oid = relnamespace
-   WHERE relkind = 'r' AND (nspname ~ '^pg_temp_') IS NOT TRUE
+   WHERE relkind = 'r' AND (nspname ~ '^pg_temp_' OR nspname ~ '^pg_deparse') IS NOT TRUE
    ORDER BY relname;
 
 -- restore normal output mode
